/*
 * *********************************************************
 * Copyright (c) 2009 - 2014, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Jul 14, 2014
 * Author(s): lukas
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.autoreferee;

import edu.dhbw.mannheim.tigers.sumatra.model.data.area.PenaltyArea;
import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.AIInfoFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.ETeamColor;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.circle.Circle;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.TrackedTigerBot;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.ids.BotID;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.AIConfig;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.TeamConfig;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.autoreferee.RefereeCaseMsg.EMsgType;


/**
 * TODO lukas, add comment!
 * - What should this type do (in one sentence)?
 * - If not intuitive: A simple example how to use this class
 * 
 * @author lukas
 */
public class MultipleDefendersCase extends ARefereeCase
{
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private ETeamColor	teamColor	= null;
	private float			margin		= 0.0f;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @param teamColor
	 */
	public MultipleDefendersCase(final ETeamColor teamColor)
	{
		this.teamColor = teamColor;
		margin = AIConfig.getGeometry().getBotRadius();
	}
	
	
	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected RefereeCaseMsg checkCase(final AIInfoFrame frame)
	{
		PenaltyArea penArea = determinePenArea(frame);
		BotID keeper = determineKeeperID();
		
		// Check each bot individually
		for (TrackedTigerBot bot : frame.getWorldFrame().getBots().values())
		{
			if (bot.getId().getTeamColor() != teamColor)
			{
				// Robot's not part of the team we're watching
				continue;
			}
			if (bot.getId().equals(keeper))
			{
				// Keeper is always allowed inside his penalty area
				continue;
			}
			
			// boolean ballInPen = penArea.isPointInShape();
			boolean ballInPen = new Circle(bot.getPos(), 120).isPointInShape(frame.getWorldFrame().getBall().getPos());
			
			// Check if fully inside the defense area
			if (penArea.isPointInShape(bot.getPos(), -margin) && ballInPen)
			{
				return new RefereeCaseMsg(teamColor, EMsgType.PENALTY_FULL);
			}
			
			if (penArea.isPointInShape(bot.getPos(), margin) && ballInPen)
			{
				return new RefereeCaseMsg(teamColor, EMsgType.PENALTY_PARTIAL);
			}
		}
		return null;
	}
	
	
	private PenaltyArea determinePenArea(final AIInfoFrame frame)
	{
		ETeamColor color = frame.getTeamColor();
		if (color == teamColor)
		{
			return AIConfig.getGeometry().getPenaltyAreaOur();
		}
		return AIConfig.getGeometry().getPenaltyAreaTheir();
	}
	
	
	private BotID determineKeeperID()
	{
		if (teamColor == ETeamColor.BLUE)
		{
			return BotID.createBotId(TeamConfig.getInstance().getTeamProps().getKeeperIdBlue(), teamColor);
		}
		return BotID.createBotId(TeamConfig.getInstance().getTeamProps().getKeeperIdYellow(), teamColor);
	}
	
	// --------------------------------------------------------------------------
	// --- getter/setter --------------------------------------------------------
	// --------------------------------------------------------------------------
}
